# -*- coding: utf-8 -*-
import logging
from typing import List, Union, Optional, Tuple

from sqlalchemy import Column, Integer, String, DateTime, func, text
from sqlalchemy.ext.declarative import declarative_base

from website.db_mysql import get_session, to_json, to_json_list, Row

"""
CREATE TABLE `enterprise_busi_model_node_device` (
  `id` int NOT NULL,
  `suid` varchar(10) DEFAULT NULL,
  `entity_suid` varchar(10) DEFAULT NULL,
  `node_id` int DEFAULT NULL,
  `node_suid` varchar(10) DEFAULT NULL,
  `busi_model_id` int DEFAULT NULL,
  `busi_model_suid` varchar(10) DEFAULT NULL,
  `device_id` int DEFAULT NULL,
  `device_suid` varchar(10) DEFAULT NULL,
  `create_time` datetime DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_0900_ai_ci COMMENT='节点-业务模型-设备关联表';
"""

Base = declarative_base()


class EnterpriseBusiModelNodeDevice(Base):
    __tablename__ = "enterprise_busi_model_node_device"

    id = Column(Integer, primary_key=True)
    suid = Column(String(10))
    entity_suid = Column(String(10))
    node_id = Column(Integer)
    node_suid = Column(String(10))
    busi_model_id = Column(Integer)
    busi_model_suid = Column(String(10))
    device_id = Column(Integer)
    device_suid = Column(String(10))
    create_time = Column(
        DateTime, default=func.current_timestamp(), onupdate=func.current_timestamp()
    )

    def __repr__(self):
        return f"EnterpriseBusiModelNodeDevice(id={self.id}, suid='{self.suid}')"


class EnterpriseBusiModelNodeDeviceRepository(object):

    def insert_record(self, records: List[EnterpriseBusiModelNodeDevice]):
        with get_session() as session:
            for record in records:
                session.add(record)
            session.commit()

    def batch_insert_record(self, records: List[dict]):
        with get_session() as session:
            session.bulk_insert_mappings(EnterpriseBusiModelNodeDevice, records)
            session.commit()

    def delete_by_busi_model_and_node(self, node_id: int, busi_model_id: int) -> bool:
        try:
            with get_session() as session:
                sql = text(
                    "delete from enterprise_busi_model_node_device "
                    "where node_id=:node_id and busi_model_id=:busi_model_id"
                )
                session.execute(
                    sql, {"node_id": node_id, "busi_model_id": busi_model_id}
                )
                session.commit()
        except Exception as e:
            logging.info("delete_by_busi_model_and_node error: %s", e)
            return False

        return True

    def check_deployed(self, node_id: int, busi_model_id: int) -> int:
        with get_session() as session:
            count = (
                session.query(EnterpriseBusiModelNodeDevice)
                .filter(
                    EnterpriseBusiModelNodeDevice.node_id == node_id,
                    EnterpriseBusiModelNodeDevice.busi_model_id == busi_model_id,
                )
                .count()
            )
            return 1 if count > 0 else 0

    def get_one_by_busimodel(self, busi_model_id: int):
        with get_session() as session:
            sql = text(
                "select node_id, busi_model_id from enterprise_busi_model_node_device "
                "where busi_model_id=:busi_model_id limit 1"
            )
            res = session.execute(sql, {"busi_model_id": busi_model_id})
            return to_json(res)

    def get_device_ids(self, node_id: int, busi_model_id: int) -> list:
        with get_session() as session:
            sql = text(
                "select device_id from enterprise_busi_model_node_device "
                "where node_id=:node_id and busi_model_id=:busi_model_id"
            )
            res = session.execute(
                sql, {"node_id": node_id, "busi_model_id": busi_model_id}
            )
            return to_json_list(res)

    def get_busi_model_by_device(self, device_id: int = 0, device_suid: str = "", pagination: bool = False,
                                 page_no: int = 0,
                                 page_size: int = 0) -> Union[list, Tuple[Optional[List[Row]], int]]:
        if not device_id and not device_suid:
            logging.error("get_busi_model_by_device error: device_id and device_suid is null")
            return []

        count = 0
        with get_session() as session:
            sql = """
            select d.busi_model_id, d.node_id, m.name, m.base_models 
            from enterprise_busi_model_node_device d, enterprise_busi_model m 
            where d.busi_model_id=m.id"""
            p = {}

            sql_count = """
            select count(1) from enterprise_busi_model_node_device d, enterprise_busi_model m where d.busi_model_id=m.id
            """
            p_count = {}

            if device_id:
                sql += " and d.device_id=:device_id"
                p.update({"device_id": device_id})

                sql_count += " and d.device_id=:device_id"
                p_count.update({"device_id": device_id})

            if device_suid:
                sql += " and d.device_suid=:device_suid"
                p.update({"device_suid": device_suid})

                sql_count += " and d.device_suid=:device_suid"
                p_count.update({"device_suid": device_suid})

            if pagination:
                sql += " order by d.id desc"

                if page_no > 0:
                    sql += " limit :pageno, :pagesize"
                    p.update({"pageno": (page_no - 1) * page_size, "pagesize": page_size})

                count = session.execute(text(sql_count), p_count).scalar()

            res = session.execute(text(sql), p)
            res = to_json_list(res)
            return res, count
